#!/usr/bin/python


"""
AnalyzeScopeData 

-------------------------------------------------------------------------------
Copyright (C) 2017 Peter Jansweijer, Henk Peek

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
-------------------------------------------------------------------------------

Usage:
  AnalyzeScopeData.py <waveform_file>
  AnalyzeScopeData.py -h | --help

  <waveform_file>     <type 'str'> file name that contains the DSO traces

Options:
  -h --help            Show this screen.
  --version            Show version.

"""
action = "standard"
# action = "length scale test"   # measurement datafile length scale factor test
# action = "sample period test"  # measurement datafile sample period scale factor test



import os
import sys
import vxi11
import scipy
import numpy
import matplotlib.pyplot as plt
import math

import pdb

# Add parent directory (containing 'lib') to the module search path
lib_path = (os.path.dirname(os.path.abspath(__file__)))
lib_path = os.path.join(lib_path,"..")
sys.path.insert(0,lib_path)

# import docopt
#import lib.LeCroy8254 as DSO

import lib.Keysight_DSO_S_254A as DSO

import lib.delay_determination as dd

directories = False
data1 = []
data2 = []
lastdirname = ""
first = True
filecount = 0
wf_file = ""

def selectfile(filelist,dirpath,dofile):
  for name in filelist:
#   print("selectfile dir name " + dirpath + "file name " + str(name))
   wf_file = os.path.join(dirpath,name)
   if os.path.isdir(wf_file):
       selectfile(os.listdir(wf_file),wf_file,dofile)
   else:
       fullfilenamepath = os.path.join(dirpath,name)
       dirname = os.path.dirname(fullfilenamepath)
       filename = os.path.basename(fullfilenamepath)
       dofile(filename,dirname)

def docount(name,dirpath):
    global directories
#    print("dowork dirname " + dirpath + "filename " + name)
#    print("dowork dirname " + os.path.dirname(os.path.join(dirpath,name)))
    if os.path.dirname(os.path.join(dirpath,name)) != "" : directories = True

def dowork(filename,dirname):      
    global lastdirname, first, filecount, data1, data2, directories, wf_file, lopt, topt
    print("dowork dirname " + dirname + "filename " + str(filename))
    if dirname != lastdirname:
      if data1 != []:
#      wf_file_dir_name = os.path.dirname(wf_file)
        result_file_data = lastdirname + " circulair-mean: " + str(numpy.mean(data1)) + \
          " se: " + str(numpy.std(data1,ddof=1)) + \
          " sem: " + str(numpy.std(data1,ddof=1)/math.sqrt(len(data1))) + \
          " lpot: " + str(lopt) + "\n" + \
          lastdirname + " lineair-mean: " + str(numpy.mean(data2)) + \
          " se: " + str(numpy.std(data2,ddof=1)) + \
          " sem: " + str(numpy.std(data2,ddof=1)/math.sqrt(len(data2))) + \
          " topt: " + str(topt) + "\n" 
        print(result_file_data)
        print("stat result file: ", os.path.join(lastdirname,"stat_result.txt"))
        result_file = open(os.path.join(lastdirname,"stat_result.txt"),"a")
        result_file.write(result_file_data)
        result_file.close()
  
        #append the results also to the result.txt file in the working directory
        # os.system('cat ' +  lastdirname + '/stat_result.txt >> result.txt')

      data1 = []
      data2 = []
      filecount = 0

    wf_file = os.path.join(dirname, filename)
    try:
      wf_data = DSO.file_to_waveform(wf_file)
      if len(wf_data) <= 1:
        print ("No, or only one oscilloscope trace in the data filea",wf_file)
        print ("Correlation is not possible!")
        return()
      else:
        print ("Oscilloscope data file contains "+str(len(wf_data))+" traces")
    except:
      print (wf_file," is no DSO file")
      return()
         
    # Take the first two available traces from the data
    trace1 = sorted(wf_data.keys())[0]
    trace2 = sorted(wf_data.keys())[1]
    print ("Correlate channels "+str(trace1)+" and "+str(trace2))

    samples = DSO.check_waveforms(wf_data)
    sample_period = wf_data[trace1]["preamble"]["x_increment"]

    x1 = wf_data[trace1]["waveform"][0][:samples]
    y1 = wf_data[trace1]["waveform"][1][:samples]
    x2 = wf_data[trace2]["waveform"][0][:samples]
    y2 = wf_data[trace2]["waveform"][1][:samples]

    # Test datafile length by divide the datafile length by "lopt"
    # "lopt" is defoult 1
    samples = samples / (lopt * topt)
    # Make sure the buffer holds an even number of samples
    a , b = divmod(samples,2)
    samples = 2 * a
    # The correlation reference point is halfway the array of samples
    correlation_zero = int(samples/2)
    # Test sample periode by multiplying by "topt"
    # "topt" is defualt 1
    #for i in range(0,samples):
    x1 = x1[::topt]
    y1 = y1[::topt]
    x2 = x2[::topt]
    y2 = y2[::topt]
    sample_period = sample_period * topt


#    doff = 8 # 200ps
#    y1d = numpy.empty_like(y1)
#    for i, y in enumerate(y1[doff:]):
#       y1d[i] = y-y1[i-doff]
#    for i in range(doff):
#       y1d[i]=y1d[doff]

#    y2d = numpy.empty_like(y2)
#    for i, y in enumerate(y2[doff:]):
#       y2d[i] = y-y2[i-doff]
#    for i in range(doff):
#       y2d[i]=y2d[doff]

    if first:
      wf_fig = plt.figure("waveform")
      ax = wf_fig.add_subplot(111)
      ax.set_xlabel('time')
      ax.set_ylabel('volatge ch:'+str(trace1)+" and "+str(trace2))
      ax.plot(x1,y1)
      ax.plot(x2,y2)
      plt.draw()

#      wf_fig = plt.figure("waveform-d")
#      ax = wf_fig.add_subplot(111)
#      ax.set_xlabel('time')
#      ax.set_ylabel('volatge ch:'+str(trace1)+" and "+str(trace2))
#      ax.plot(x1,y1d)
#      ax.plot(x2,y2d)
#      ax.plot(x1,y1d**3)
#      ax.plot(x2,y2d**3)
#      plt.draw()

    cx, cy = dd.correlate_circular(y1, y2)

    cor_ref = int(samples/2)
    estimated_max = numpy.argmax(cy) - correlation_zero

#    print ("estimated max raw type: ",type(estimated_max))
# if emstimated_max is more + 3 ns: wrong correlation: add 2**7-1 (=PRBS-7) gigabit periods (= 800 ps)
    if estimated_max < -400/topt:     # -12E-9 / sample_period:
         estimated_max = numpy.int64(estimated_max + ((2**7-1) * 0.8E-9)/(sample_period*topt))  # (2**7-1) * 0.8E-9 / sample_period
#    print ("estimated max type: ",type(estimated_max),estimated_max)

    delay1 = sample_period * dd.peak_position(cx, cy, x_estimate = estimated_max)
    cor=50
    cory=numpy.correlate(y1[estimated_max:],y2[cor:-estimated_max-cor],mode="valid")
    corx=numpy.arange(estimated_max -cor,estimated_max + cor+1 )
    cor_estimated_max=numpy.argmax(cory)+estimated_max-cor
#    print("main convole cory: ",type(cory),cory)
    print("main convole cor_estimated_max: ",type(cor_estimated_max),cor_estimated_max) 
    delay2 = sample_period * dd.peak_position(corx, cory, x_estimate = estimated_max)
    print ("circulair delay1: ", delay1, "lineair delay2: ", delay2)
#    plt.plot(corx,cory)
#    plt.plot(corx,y1[:10001])
#    plt.plot(corx,y2[:10001])
#    plt.draw()
#    plt.show()
#    pdb.set_trace()
#    zx, zy = dd.correlate_linear(y1, y2, center = estimated_max)
#    print("main lineair correlate zx: ", type(zx),zx)
#    print("main lineair correlate zy: ", type(zy),zy)
#    z_estimated_max = numpy.argmax(zy) - correlation_zero
#    print("main lineair correlate z_estimated_max: ", type (z_estimated_max),z_estimated_max)
#    delay2 = sample_period * dd.peak_position(zx, zy, x_estimate = z_estimated_max)

    print(wf_file + " estimated circulair sample max: " + str(estimated_max) + " circulair delay " + str(delay1))
    if directories:
#      print("result file: ", os.path.join(dirname,"result.txt"))
      result_file = open(os.path.join(dirname,"result.txt"),"a")
      result_file.write(str(filecount) + "  " +wf_file + ": circulair " + str(delay1) + " lineair  " + str(delay2) + "  " + "\n")
      result_file.close()

      data1.append(delay1)
      data2.append(delay2)



    if first:
      first = False
      cor_fig = plt.figure("correlation")
      ax = cor_fig.add_subplot(111)
      ax.text(0.01,0.95,'correlation of ch: '+str(trace1)+' and '+str(trace2), transform=ax.transAxes)
      ax.text(0.01,0.90,'delay at estimated max {0:f} = {1:.6g}'.format(estimated_max,delay1), transform=ax.transAxes)
      ax.set_xlabel('sample')
      ax.set_ylabel('correlation')
      ax.plot(cx,cy)
      plt.draw()
 
#      cor_fig = plt.figure("correlation-d")
#      ax = cor_fig.add_subplot(111)
#      ax.set_xlabel('sample')
#      ax.set_ylabel('correlationi')
#      ax.plot(cdx,cdy)
#      plt.draw()

    lastdirname = dirname
    filecount += 1

############################################################################
##
## If run from commandline, we can test the library
##

if __name__ == "__main__":
    selectfile(sys.argv[1:],"",docount)
#    print("directories " + str(directories))
    lopt = 1    # measurement datafile length scale factor
    topt = 1    # measurement datafile sample period scale factor

    if action == "standard":
      selectfile(sys.argv[1:],"",dowork)
      dowork("","")    #dummy call to generate last result stat file

    if action == "length scale test":
     for lopt in [1,2,4,8,16,32,64]: 
      selectfile(sys.argv[1:],"",dowork)
      dowork("","")    #dummy call to generate last result stat file

    if action == "sample period test":
     for topt in [1,2,3,4,5,6,7,8,10,12,14,16,20,24,28,32,40]:    
      selectfile(sys.argv[1:],"",dowork)
      dowork("","")    #dummy call to generate last result stat file

    plt.show()

#  working_directory_result_file=open("result.tst","a")
#  working_directory_result_file.write(result_file_data)
#  os.system('cat ' +  dir_name_0 + '/data*/stat_result.txt >> ' + dir_name_0 + '/result.txt')
#  working_directory_result_file.close()




